@@ -28,6 +28,7 @@ class XRaySettingsConfigParser:
             self.patch_modules = xray.get("PATCH_MODULES", None)
             self.logging_enabled = xray.get("LOGGING_ENABLED", False)
             self.sqlalchemy = xray.get("SQLALCHEMY", False)
+            self.middleware = xray.get("MIDDLEWARE", True)
         else:
             self.enabled = False
             self.service_name = None
@@ -38,6 +39,7 @@ class XRaySettingsConfigParser:
             self.patch_modules = None
             self.logging_enabled = False
             self.sqlalchemy = False
+            self.middleware = True
 
 
 class XRaySettingsLazySettings:
@@ -51,6 +53,7 @@ class XRaySettingsLazySettings:
         self.patch_modules = settings.XRAY_PATCH_MODULES
         self.logging_enabled = settings.XRAY_LOGGING_ENABLED
         self.sqlalchemy = settings.XRAY_SQLALCHEMY
+        self.middleware = settings.XRAY_MIDDLEWARE
 
 
 class XRayConfigurator:
@@ -69,7 +72,8 @@ class XRayConfigurator:
             if self.settings.sqlalchemy:
                 logging.info("XRayFlaskSqlAlchemy enabled")
                 XRayFlaskSqlAlchemy(application)
-            XRayMiddleware(application, xray_recorder)
+            if self.settings.middleware:
+                XRayMiddleware(application, xray_recorder)
 
     def _init_xray(self, settings):
         if settings.enabled:
@@ -101,7 +105,7 @@ class XRayConfigurator:
 def task_prerun(task_id, task, *args, **kwargs):
     logging.info("task prerun task_id: " + task_id)
     xray_header = construct_xray_header(task.request)
-    print(xray_header.root + " " + xray_header.parent)
+    logging.info("xray header constructed. Root: {}, Parent: {}".format(xray_header.root, xray_header.parent))
     segment = xray_recorder.begin_segment(
         name=task.name,
         traceid=xray_header.root,
@@ -120,14 +124,13 @@ def task_postrun(task_id, *args, **kwargs):
 @signals.before_task_publish.connect
 def before_task_publish(sender, headers, **kwargs):
     logging.info("before taskpublish sender: " + sender)
-    current_segment = xray_recorder.current_segment()
     subsegment = xray_recorder.begin_subsegment(
         name=sender,
         namespace='remote',
     )
 
     if subsegment is None:
-        # Not in segment
+        logging.info("task taskpublish not in segment")
         return
     subsegment.put_metadata('task_id', headers.get("id"), namespace='celery')
     inject_trace_header(headers, subsegment)
@@ -143,6 +146,9 @@ def xray_after_task_publish(**kwargs):
 def xray_task_failure(einfo, **kwargs):
     logging.info("task_failure")
     segment = xray_recorder.current_segment()
+    if segment is None:
+        logging.info("task failure not in segment")
+        return
     if einfo:
         stack = stacktrace.get_stacktrace(limit=xray_recorder.max_trace_back)
         segment.add_exception(einfo.exception, stack)
